#include "myfndt.h"
#include "casort.h"

const int CCacheSorter::DefaultTargetBlockSize = 600;
/* profile 28. 12. 96: cache 1500
   300      13 min 
   600      12 min 
   1000     13 min */

CCacheSorter::CCacheSorter(CReporter *aReporter,
    CInQueue &aInQueue,
    COutQueue &aOutQueue,
    int aTargetBlockSize):
    CTransmission<CSimpleOcc, COccBlock>(aReporter, aInQueue,
			aOutQueue),
    OccBlock(NULL),
    TargetBlockSize(aTargetBlockSize)
{
}

CCacheSorter::~CCacheSorter()
{
	delete OccBlock;
}

short CCacheSorter::IsFilled()
{
    if (!IsSourceDone())
        return OccBlock != NULL;
    else
	   return !WordCache.IsEmpty();
}

short CCacheSorter::GetPerc()
{
    return 50; // not used anyway
}

void CCacheSorter::SetIn(CSimpleOcc *InOcc)
{   
    CObjPtr<CComposedOcc> ComposedOcc = WordCache.NewMerge(InOcc);
    Assert(OccBlock == NULL);
    if (ComposedOcc != NULL)
    {    OccBlock = new COccBlock(TargetBlockSize);
	    OccBlock->SetIn(ComposedOcc.Drop());
	    FillBlock(OccBlock);
    }
}

COccBlock *CCacheSorter::NewOut()
{
    if (!IsSourceDone())
    {    Assert(OccBlock != NULL);
	    COccBlock *OutOccBlock = OccBlock;
	    OccBlock = NULL;
	    return OutOccBlock;
    }
    else
    {    Assert(OccBlock == NULL);
	    COccBlock *OutOccBlock = new COccBlock(TargetBlockSize);
	    CComposedOcc *ComposedOcc = WordCache.YankFirst();
	    Assert(ComposedOcc != NULL);
	    OutOccBlock->SetIn(ComposedOcc);
	    FillBlock(OutOccBlock);
	    return OutOccBlock;
    }
}

void CCacheSorter::FillBlock(COccBlock *OutBlock)
{
    for (int i = 0; i < TargetBlockSize - 1; i++)
    {    CComposedOcc *Neighbor = WordCache.YankNeighbor();
	    if (!Neighbor)
		   return;
	    OutBlock->SetIn(Neighbor);
    }
}



