#include "myfndt.h"
#include "occmap.h"
#include "abststem.h"

const int COccBlock::OccColMax = 8000;
    // OccColMax * sizeof(long) < 0xffff / 2
    // COccBuilder copies occurences into memory whose length is short

COccBlock::COccBlock(int aSize):
    Emptying(0),
    PhaseCount(0),
    Itr(begin()),
    BigCol(0),
    AliasCount(0)
{
}

COccBlock::~COccBlock()
{
}

int COccBlock::IsEmpty() const
{
    return Emptying ? !PhaseCount : DMap<DString, COccCol>::IsEmpty();
}
  
int COccBlock::GetCount() const
{
    return Emptying ? PhaseCount : DMap<DString, COccCol>::GetCount();
}

void COccBlock::SetIn(CSimpleOcc *InSimpleOcc,
    CAbstractStemmer *Stemmer)
{
    Assert(!Emptying);
    CObjPtr<CSimpleOcc> SimpleOcc = InSimpleOcc;
    DMap<DString, COccCol>::iterator i = find(InSimpleOcc->OccWord);

    if (!(i == end()))
        AddToOld((*i).second, InSimpleOcc->GetData());
    else
    {   DString NWord = InSimpleOcc->OccWord;
        if (Stemmer->Apply(NWord) ||
		    NWord == InSimpleOcc->OccWord)
        {   COccCol NMaster(1);
            NMaster.Add(InSimpleOcc->GetData());
            Add(InSimpleOcc->OccWord, NMaster);
        }
        else
        {   DMap<DString, COccCol>::iterator j = find(NWord);
            if (!(j == end()))
            {   AddToOld((*j).second, InSimpleOcc->GetData());
                COccCol NAlias((*j).second, 0);
                Add(InSimpleOcc->OccWord, NAlias);
            }
            else
            {   COccCol NMaster(1);
                NMaster.Add(InSimpleOcc->GetData());
                Add(NWord, NMaster);
                COccCol NAlias(NMaster, 0);
                Add(InSimpleOcc->OccWord, NAlias);
            }
            ++AliasCount;
        }
    }
}

void COccBlock::AddToOld(COccCol &OldCol, ulong Data)
{
    OldCol.Add(Data);
    if (OldCol.GetOccs().size() > OccColMax)
    {   /* profile 14Jun98: checking for stop words here
            does not measurably improve performance */
        BigCol = 1;
    }
}

CComposedOcc *COccBlock::NewComposedOcc()
{
    if (!Emptying)
    {   Emptying = 1;
        PhaseCount = DMap<DString, COccCol>::GetCount() - AliasCount;
        Itr = begin();
    }

    Assert(!(Itr == end()));
    while (!(*Itr).second.IsMaster())
    {   ++Itr;
	   Assert(!(Itr == end()));
    }

    Assert(PhaseCount > 0);
    --PhaseCount;
    CComposedOcc *ComposedOcc = new CComposedOcc((*Itr).first, (*Itr).second);
    ++Itr; 
    return ComposedOcc;
}
